/*
 * Copyright (c) 2007-2010 Koji Suzuki
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>

#define bit_set(port,bit)	{ port |= (1<<(bit));}
#define bit_clear(port,bit)	{ port &= ~(1<<(bit));}
#define min(a,b)		((a)<(b) ? (a):(b))

#define nop()	__asm__ volatile ("nop" )
#define sleep()	__asm__ volatile ("sleep" )

#define BUTTON_SUPPORT
#ifndef TIFR0
#define TIFR0	TIFR
#endif

#include "button.h"
#include "i2cdev.h"
#include "lcd.h"

uint8_t button_reg;

int main(void)
{
    int8_t r;
    uint8_t rs,d;

    /* set system_clock pre scaler 1/8 => 1/1 */
    CLKPR = (1<<CLKPCE);
    CLKPR = 0;

    lcd_init();
    i2cdev_init(0x3e);
    //usart_init();
    //init_i2c();

#ifdef BUTTON_SUPPORT
    TCCR0B = 0x4; // 1/256 free running
    i2cdev_set_rx_buf(&button_reg, 1);
#endif
    for(;;) {

	r = i2cdev_can_getc();
	if (r <0) {
	    i2cdev_flush();
	} else if (r) {
	    rs = i2cdev_getc() & 0x40;
	    if (i2cdev_can_getc() > 0) {
		d = i2cdev_getc();
		if (rs) lcd_data(d);
		else {
		    if ((d & 0xe0) == 0x20) {	// Function Set
			d &= ~0x10;		// 4 bit-mode only
		    }
		    lcd_cmd(d);
		}
	    }
	}
#ifdef BUTTON_SUPPORT
	if (bit_is_set(TIFR0, TOV0)) {
		button_reg = check_button() & 0x7;
		bit_set(TIFR0, TOV0);
	}
#endif
    }
    return 0;
}
