/*
 * Copyright (c) 2010 Koji Suzuki
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>

#define bit_set(port,bit)	{ port |= (1<<(bit));}
#define bit_clear(port,bit)	{ port &= ~(1<<(bit));}
#define min(a,b)		((a)<(b) ? (a):(b))

#define nop()	__asm__ volatile ("nop" )
#define sleep()	__asm__ volatile ("sleep" )



/*
SC802A-3.3V

    +3.3V        NC NC          SW -- GND
    VDD  RS  E  DB1 DB3 DB5 DB7 BLK
  2  o   o   o   o   o   o   o   o  16  
  1  o   o   o   o   o   o   o   o  15  
    VSS  Vo R/W DB0 DB2 DB4 DB6 BLA
    GND GND?     NC NC          +5V
*/

#define LCD_PORT PORTB
#define LCD_DDR  DDRB

#define LCD_E_PORT PORTB
#define LCD_E_DDR  DDRB

#define LCD_RS_PORT PORTD
#define LCD_RS_DDR  DDRD

#define LCD_E    PB6
#define LCD_RS   PD5
#define LCD_DB7  PB3
#define LCD_DB6  PB2
#define LCD_DB5  PB1
#define LCD_DB4  PB0

#define LCD_MASK ((1<<LCD_DB7) |(1<<LCD_DB6) |(1<<LCD_DB5) |(1<<LCD_DB4) )
#define LCD_SHIFT LCD_DB4

static void __lcd_out(uint8_t cd, uint8_t data) {
	if (!cd) {
		bit_set(LCD_RS_PORT, LCD_RS);
	} else {
		bit_clear(LCD_RS_PORT, LCD_RS);
	}
	data &= 0xf;
	data <<= LCD_SHIFT;
	LCD_PORT = (LCD_PORT & ~LCD_MASK) | data ;
	_delay_us(1);
	bit_set(LCD_E_PORT, LCD_E);
	_delay_us(1);
	bit_clear(LCD_E_PORT, LCD_E);
}

void lcd_cmd(uint8_t data) {
	__lcd_out(1, data >> 4);
	__lcd_out(1, data & 0xf);
	_delay_us(50);
}

void lcd_data(uint8_t data) {
	__lcd_out(0, data >> 4);
	__lcd_out(0, data & 0xf);
	_delay_us(60);
}

void lcd_init() {
	LCD_DDR = LCD_MASK;
	bit_set(LCD_RS_DDR,LCD_RS);
	bit_clear(LCD_E_PORT, LCD_E);
	bit_set(LCD_E_DDR, LCD_E);
	_delay_ms(50);
	__lcd_out(0, 0x2); // 4bit mode
	_delay_us(50);
	lcd_cmd(0x38); // 2 line mode
	lcd_cmd(0x0c); // display on, cursor off, blink off
	lcd_cmd(0x01); // display clear
	_delay_ms(2);
	lcd_cmd(0x06); // auto inc, entire shift off
}

void lcd_move(int8_t line,int8_t col) {
	uint8_t cmd;
	cmd = 0x80 | (line?0x40:0x00) | col;
	lcd_cmd(cmd); // set ddram addr
}
